<?php

/**
 * @file
 * Theme for Gantt views.
 */

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Link;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;

/**
 * Template preprocess views gantt.
 *
 * @param array $variables
 *   Array variable.
 */
function template_preprocess_views_view_gantt(array &$variables) {
  $view = $variables['view'];
  $style = $view->style_plugin;
  $setting = $style->options;
  // Entity info form view.
  $filter = $view->filter;
  $setting['view_type_entity'] = $view->getBaseEntityType()->id();
  $setting['view_bundle_entity'] = key($filter['type']->value);

  $fieldDate = $view->getHandlers('field')[$setting["start_date"]];
  $pluginManage = \Drupal::entityTypeManager();
  $dateType = $pluginManage
    ->getStorage('date_format')
    ->load($fieldDate["settings"]["format_type"]);
  $entityReferenceSelection = Drupal::service('plugin.manager.entity_reference_selection');
  $patterns = !empty($dateType) ? explode(' ', $dateType->getPattern()) : ['d-m-Y'];
  $patterns = array_filter(
    $patterns,
    function ($value) {
      return strlen($value) >= 4;
    }
  );
  $setting['format_date'] = current($patterns);
  $variables["view_id"] = $view->storage->id();
  $variables["display_id"] = $view->current_display;
  if ($setting["cdn"]) {
    $variables['view']->element['#attached']['library'][] = 'gantt/gantt.cdn';
  }
  else {
    $variables['view']->element['#attached']['library'][] = 'gantt/gantt';
  }

  // $fieldDefinitions && $entity.
  if (!empty($variables['rows'])) {
    $firstElement = current($variables['rows']);
    if (!empty($firstElement->_entity)) {
      $entity = $firstElement->_entity;
      $fieldDefinitions = $entity->getFieldDefinitions();
    }
  }

  // Get list owner.
  $variables["owner"] = [];
  if (!empty($setting['owner']) && !empty($entity) && !empty($fieldDefinitions)) {
    $entityDraft = $pluginManage->getStorage($entity->getEntityTypeId())->create(['type' => $entity->bundle()]);
    $fieldDefinitions = $entityDraft->getFieldDefinitions();
    $listOwners = $entityReferenceSelection
      ->getSelectionHandler($fieldDefinitions[$setting['owner']], $entityDraft)
      ->getReferenceableEntities();
    if (!empty($listOwners['user'])) {
      $users = $listOwners['user'];
      if (!empty($users[0])) {
        unset($users[0]);
      }
      $colorIndex = 0;
      foreach ($users as $uid => $userName) {
        $variables["owner"][] = [
          'key' => $uid,
          'label' => $userName,
          'textColor' => "#FFF",
          'backgroundColor' => gantt_generate_color($colorIndex++),
        ];
      }
    }
  }

  // Get list custom resource.
  $fieldDefinitions = [];
  if (!empty($setting['custom_resource']) && !empty($entity)) {
    $listSource = [];
    $list_source = [];
    $sourceBundleMapping = [];
    $setting_custom_resource = [];
    $group_resource = [];
    if ($setting['group_resource']) {
      $group_resource[] = [
        "name" => t("Default"),
        "field_resource" => t("default"),
      ];
    };
    $entityDraft = $pluginManage->getStorage($entity->getEntityTypeId())->create(['type' => $entity->bundle()]);
    $fieldDefinitions = $entityDraft->getFieldDefinitions();
    foreach ($setting['custom_resource'] as $field) {
      $sourceBundleMapping[$field] = $fieldDefinitions[$field]->getSetting('handler_settings')['target_bundles'] ?? ["user" => "user"];
      if ($field != $setting['owner']) {
        if (in_array($field, $setting['group_resource'])) {
          $group_resource[] = [
            "name" => $fieldDefinitions[$field]->getLabel(),
            "field_resource" => $field,
          ];
        }
        $setting_custom_resource[] = $field;
        $listSource[$field] = $entityReferenceSelection->getSelectionHandler($fieldDefinitions[$field], $entityDraft)
          ->getReferenceableEntities();
      }
    }

    if (!empty($sourceBundleMapping)) {
      foreach ($sourceBundleMapping as $field => $bundleLinks) {
        $list_source[$field] = [];
        if (!empty($bundleLinks)) {
          foreach ($bundleLinks as $bundleName) {
            if (!empty($listSource[$field][$bundleName])) {
              foreach ($listSource[$field][$bundleName] as $id => $name) {
                $list_source[$field][] = [
                  'key' => $id,
                  'label' => $name,
                ];
              }
            }
          }
        }
      }
    }
  }

  // Label resource.
  $resource_label = [];
  if (!empty($setting['custom_resource'])) {
    foreach ($setting['custom_resource'] as $resource) {
      $resource_label[$resource] = $fieldDefinitions[$resource]->getLabel();
    }
  }

  // Custom field.
  if (!empty($setting['custom_field']) && !empty($fieldDefinitions[$setting['custom_field']])) {
    $setting['custom_field_name'] = $fieldDefinitions[$setting['custom_field']]->getLabel();
  }
  // Order field.
  if (!empty($setting['order_field'])) {
    $setting['order_field_name'] = $setting['order_field'];
  }
  // Convert holiday to format Y-m-d.
  if (!empty($setting['holidays'])) {
    $current_year = date("Y");
    $new_date_array = [];
    foreach (explode(',', str_replace(' ', '', $setting['holidays'])) as $holiday) {
      $extract = explode('-', $holiday);
      $day = $extract[0];
      $month = $extract[1];
      $year = $extract[2] ?? $current_year;
      $new_date_array[] = date('Y-m-d', strtotime("$year-$month-$day"));
    }
    $setting['holidays'] = $new_date_array;
  }

  $id = implode('-', ["gantt", $view->id(), $view->current_display]);
  $variables["id_gantt"] = $id;
  $variables["toolbar_option"] = [
    'auto_schedule' => $setting['auto_schedule'],
    'group_resource' => !empty($group_resource) ? $group_resource : [],
  ];
  $variables["settings"] = [
    'use_cdn' => $setting['cdn'],
  ];
  $currentPath = \Drupal::service('path.current')->getPath();
  $destination = [
    'query' => ['destination' => $currentPath],
  ];
  $variables['view']->element['#attached']['drupalSettings']['gantt'][$id] = [
    'id' => $id,
    'owner' => $setting['owner'],
    'date_format' => str_replace(
      ['Y', 'm', 'd', 'H', 'i', 's'],
      ['%Y', '%m', '%d', '%H', '%i', '%s'],
      $setting['format_date']
    ),
    'custom_resource' => [
      'resource' => !empty($setting_custom_resource) ? $setting_custom_resource : [],
      'resource_label' => $resource_label,
      'resource_column' => array_values($setting['show_column_resource']) ?? [],
      'resource_lightbox' => array_values($setting['show_lightbox_resource']) ?? [],
      'resource_server_list' => !empty($list_source) ? $list_source : [],
    ],
    'ajax' => Url::fromRoute('gantt.ajax', [
      'view_id' => $variables["view_id"],
      'display_id' => $variables["display_id"],
    ])->toString(),
    'import' => Url::fromRoute('gantt.import', [
      'view_id' => $variables["view_id"],
      'display_id' => $variables["display_id"],
    ])->toString(),
    'custom_field' => $setting['custom_field'] ?? FALSE,
    'custom_field_name' => $setting['custom_field_name'] ?? FALSE,
    'current_path' => 'destination=' . $currentPath,
    'add_task' => $setting["add_task"],
    'edit_task' => $setting["edit_task"],
    'native_dialog' => $setting["native_dialog"],
    'add_link' => '',
    'edit_link' => '',
    'work_time' => !empty($setting['work_time']),
    'work_day' => $setting['work_day'],
    'has_duration' => !empty($setting['duration']),
    'has_planned' => FALSE,
    'has_permission' => \Drupal::currentUser()->hasPermission('administer views gantt'),
    'show_button_detail' => $setting['show_button_detail'],
    'permission_edit' => $setting['permission_edit'],
    'creator' => $setting['creator'],
    'holidays' => $setting['holidays'],
    'use_cdn' => $setting['cdn'],
    'auto_schedule' => $setting['auto_schedule'],
    'group_field' => $setting['group_field'],
    'last_of_the_day' => $setting['last_of_the_day'],
    'select_parent' => $setting['select_parent'],
  ];
  $variables['resource'] = !empty($list_source) ? $list_source : [];
  $variables["add_link"] = '';
  if (!empty($variables['rows'])) {
    $entityFirst = current($variables['rows'])->_entity;
    $setting['entity_type'] = $entityFirst ? $entityFirst->getEntityTypeId() : NULL;
    if ($setting["native_dialog"]) {
      $addLinkText = Markup::create('<i class="bi bi-plus-lg"></i> ' . t('Add'));

      if (!empty($entityFirst)) {
        $entity_type = $entityFirst->getEntityTypeId();
        $entity_bundle = $entityFirst->bundle();
        if ($entity_type == 'node') {
          $uriAdd = Url::fromRoute($setting['view_type_entity'] . '.add', [
            'node_type' => $entity_bundle,
          ], $destination);
        }
        if ($entity_type == 'paragraph') {
          $uriAdd = Url::fromRoute('gantt.add', [
            'paragraph_type' => $entity_bundle,
            'entity_type' => $entityFirst->get('parent_type')->value,
            'entity_field' => $entityFirst->get('parent_field_name')->value,
            'entity_id' => $entityFirst->get('parent_id')->value,
          ], $destination);
        }
      }

      if ($setting["add_task"]) {
        if (!empty($uriAdd)) {
          $variables["add_link"] = Link::fromTextAndUrl($addLinkText, $uriAdd)
            ->toString();
          $variables['view']->element['#attached']['drupalSettings']['gantt'][$id]["add_link"] = $uriAdd->toString();
        }
      }
    }
  }
  if (!empty($setting['baseline']['planned_date'])) {
    $setting['planned_start'] = $setting['planned_end'] = $setting['baseline']['planned_date'];
    if (!$setting['baseline']['split_actual_planned']) {
      $variables['view']->element['#attached']['drupalSettings']['gantt'][$id]["has_planned"] = TRUE;
    }
  }
  $data = convert_gantt_data($variables['rows'], $setting, $destination, $view);
  $variables["data"]["data"] = $data['data'];
  $variables["dataPlanned"] = !empty($data['planned']) ? ['data' => $data['planned']] : [];
  if (!empty($data['links'])) {
    $variables["data"]['links'] = $data['links'];
    if (!empty($variables["dataPlanned"])) {
      $variables["dataPlanned"]['links'] = $data['links'];
    }
  }
  if (!empty($data['marker'])) {
    $variables["marker"] = $data['marker'];
  }
  if (!empty($setting['owner']) || !empty($setting['custom_resource'])) {
    $variables['view']->element['#attached']['library'][] = 'gantt/gantt.chosen';
  }

  // Set session variable for import paragraphs.
  if (!empty($view->argument)) {
    foreach ($view->argument as $key => $arg) {
      if ($key == 'parent_id') {
        $variables["paragraph_parent_id"] = $view->args[$arg->position];
        if (!empty($entityFirst)) {
          if (!empty($entityFirst->parent_type)) {
            $variables["paragraph_parent_type"] = $entityFirst->parent_type->value;
          }
          if (!empty($entityFirst->parent_field_name)) {
            $variables["paragraph_parent_field_name"] = $entityFirst->parent_field_name->value;
          }
        }
      }
    }
  }
}

/**
 * Change color backgroup foreach user.
 *
 * @param int $index
 *   Index user in array.
 *
 * @return string
 *   Code color.
 */
function gantt_generate_color(int $index) {
  $listColor = ["#03A9F4", "#f57730", "#e157de", "#78909C", "#8D6E63",
    "#800000", "#008080", "#d2691e", "#ff8c00", "#daa520", "#4b0082",
    "#FF0000", "#800080", "#008000", "#808000", "#000080", "#0000FF",
  ];
  if (empty($listColor[$index])) {
    return $listColor[$index % count($listColor)];
  }
  return $listColor[$index];
}

/**
 * {@inheritDoc}
 */
function convert_gantt_data($items, array $setting, $destination, $view = FALSE) {
  $links = [];
  $outData = [];
  $mappingGantt = [
    'text',
    'type',
    'start_date',
    'duration',
    'progress',
    'open',
    'parent',
    'owner',
    'planned_start',
    'planned_end',
    'custom_field',
    'creator',
    'custom_resource',
    'resource_has_edit',
  ];
  $fieldDateType = 'date';
  if (!empty($setting['group_field'])) {
    $bundle_info = \Drupal::service('entity_type.bundle.info');
    $entity_types = $bundle_info->getBundleInfo($setting['entity_type']);
    $groupLabel = [];
  }
  $minDate = [];
  $maxDate = [];
  $emptyDate = [];
  $data = [];
  foreach ($items as $delta => $item) {
    if (empty($item->_entity)) {
      continue;
    }
    $entity = $item->_entity;
    $group = 0;
    if (!empty($setting['group_field'])) {
      $group = $entity->get($setting["group_field"])->getString();
      if (empty($groupLabel[$group])) {
        $view->row_index = $delta;
        $groupLabel[$group] = strip_tags($view->field[$setting["group_field"]]->advancedRender($item));
      }
    }
    $fieldDefinitions = $entity->getFieldDefinitions();
    if (!empty($fieldDefinitions[$setting['start_date']])) {
      $fieldDateType = $fieldDefinitions[$setting['start_date']]->getType();
    }
    $data[$group][$delta] = ['id' => $entity->id()];
    $planned = [];
    foreach ($mappingGantt as $ganttField) {
      $field_name = $setting[$ganttField] ?? '';
      if (!empty($setting[$ganttField]) && !is_array($setting[$ganttField]) && !empty($entity->$field_name)) {
        $value = $entity->get($field_name)->getString();
        if (!empty($value)) {
          $data[$group][$delta][$ganttField] = $value;
        }
        $start_date = NULL;
        if ($ganttField == 'start_date') {
          if (!empty($entity->get($field_name)->date)) {
            $start_date = $entity->get($field_name)->date->format($setting['format_date']);
          }
          if ($fieldDateType == 'daterange' && $dateValue = $entity->get($field_name)->value) {
            $start_date = date($setting['format_date'], strtotime($dateValue));
          }
          if (!empty($start_date)) {
            $start_date = \DateTime::createFromFormat($setting['format_date'], $start_date);
            $minDate[] = $start_date->getTimestamp();
            $data[$group][$delta][$ganttField] = $start_date->format($setting['format_date']);
            $start_date = $start_date->format('Y-m-d');
          }
          else {
            $emptyDate[] = [
              'group' => $group,
              'delta' => $delta,
              'field' => $ganttField,
            ];
          }
        }
        if ($ganttField == 'duration' && !empty($start_date)) {
          $maxDate[] = strtotime("$start_date +$value days");
        }

        // Process owner.
        if ($ganttField == 'owner') {
          $data[$group][$delta][$ganttField] = [];
          if (!$entity->get($field_name)->isEmpty()) {
            $values = $entity->get($field_name)->getValue();
            foreach ($values as $value) {
              if (!empty($value->entity)) {
                $data[$group][$delta][$ganttField][] = $value->entity->id();
              }
              if (!empty($value["target_id"])) {
                $data[$group][$delta][$ganttField][] = $value["target_id"];
              }
            }
          }
        }

        // Process custom field.
        if ($ganttField == 'custom_field') {
          $data[$group][$delta][$field_name] = $value;
        }

        // Process planned date.
        if ($ganttField == 'planned_start') {
          if (!empty($entity->get($field_name)->date)) {
            $data[$group][$delta][$ganttField] = $entity->get($field_name)->date->format($setting['format_date']);
          }
          if ($fieldDateType == 'daterange' && $dateValue = $entity->get($field_name)->value) {
            $dateValue = new DrupalDateTime($dateValue);
            $data[$group][$delta][$ganttField] = $dateValue->format($setting['format_date']);
          }
        }
        if ($ganttField == 'planned_end') {
          if (!empty($entity->get($field_name)->date)) {
            $data[$group][$delta][$ganttField] = $entity->get($field_name)->date->format($setting['format_date']);
          }
          if ($fieldDateType == 'daterange' && $dateValue = $entity->get($field_name)->end_value) {
            $dateValue = new DrupalDateTime($dateValue);
            $data[$group][$delta][$ganttField] = $dateValue->format($setting['format_date']);
            // Option Last of the day.
            if ($setting['last_of_the_day']) {
              $dateValue->modify('+1 day');
              $data[$group][$delta]['planned_end'] = $dateValue->format($setting['format_date']);
            }
          }
        }

        // Process creator.
        if ($ganttField == 'creator') {
          $data[$group][$delta][$ganttField] = [];
          $creators = !$entity->get($field_name)
            ->isEmpty() ? $entity->get($field_name)->getValue() : [];
          if ($creators) {
            foreach ($creators as $index => $value) {
              $creator = $entity->get($field_name)->referencedEntities();
              $data[$group][$delta][$ganttField][] = $value["target_id"];
              $data[$group][$delta]["creator_name"][] = $creator[$index]->getDisplayName();
            }
          }
        }
      }

      // Process resource.
      if ($ganttField == 'custom_resource') {
        foreach ($setting[$ganttField] as $item) {
          $data[$group][$delta][$item] = [];
          if (!$entity->get($item)->isEmpty()) {
            $values = $entity->get($item)->getValue();
            foreach ($values as $value) {
              if (!empty($value->entity)) {
                $data[$group][$delta][$item][] = $value->entity->id();
              }
              elseif (is_array($value) && !empty($value['target_id'])) {
                $data[$group][$delta][$item][] = $value['target_id'];
              }
            }
          }
        }
      }

      // Link detail of task.
      if ($setting['show_button_detail']) {
        if ($entity->getEntityTypeId() == "node") {
          $data[$group][$delta]["link_detail"] = $entity->toUrl("edit-form")->setOptions($destination)->toString();
        }
        if ($entity->getEntityTypeId() == "paragraph") {
          $data[$group][$delta]["link_detail"] = Url::fromRoute('gantt.edit', ['paragraph' => $entity->id()])
            ->setOptions($destination)->toString();
        }
      }

      // Resource edit.
      if (!empty($setting['permission_edit']) && $ganttField == 'resource_has_edit' && !empty($setting[$ganttField])) {
        $data[$group][$delta]['has_edit'] = [];
        foreach ($setting[$ganttField] as $item) {
          if (!$entity->get($item)->isEmpty()) {
            $values = $entity->get($item)->getValue();
            foreach ($values as $value) {
              if (!empty($value->entity)) {
                $data[$group][$delta]['has_edit'][] = $value->entity->id();
              }
              elseif (is_array($value) && !empty($value['target_id'])) {
                $data[$group][$delta]['has_edit'][] = $value['target_id'];
              }
            }
          }
        }
        if (!in_array($view->getUser()->id(), $data[$group][$delta]['has_edit'])) {
          $data[$group][$delta]['readonly'] = TRUE;
          if ($setting['show_button_detail']) {
            if ($entity->getEntityTypeId() == "node") {
              $data[$group][$delta]["link_detail"] = $entity->toUrl("canonical")->setOptions($destination)->toString();
            }
            if ($entity->getEntityTypeId() == "paragraph") {
              $data[$group][$delta]["link_detail"] = Url::fromRoute('gantt.display', ['paragraph' => $entity->id()])
                ->setOptions($destination)->toString();
            }
          }
        }
      }
    }

    // Data planned.
    if (!empty($setting['baseline']['split_actual_planned']) && !empty($field_name = $setting['baseline']['planned_date']) && !empty($fieldDefinitions[$setting['baseline']['planned_date']])) {
      $planned[$group][$delta] = $data[$group][$delta];
      $planned[$group][$delta]['start_date'] = NULL;
      $planned[$group][$delta]['duration'] = 1;

      if (!empty($entity->get($field_name)->date)) {
        $data[$group][$delta]['start_date'] = $entity->get($field_name)->date->format('d-m-Y H:i');
      }
      $fieldPlannedDateType = $fieldDefinitions[$field_name]->getType();
      if ($fieldPlannedDateType == 'daterange' && $dateValue = $entity->get($field_name)->value) {
        $planned[$group][$delta]['start_date'] = date('d-m-Y H:i', strtotime($dateValue));
      }
      if (!empty($field_name = $setting['baseline']['planned_duration'])) {
        $value = $entity->get($field_name)->getString();
        if (!empty($value)) {
          $planned[$group][$delta]['duration'] = $value;
        }
      }
      if (!empty($planned[$group][$delta]) && empty($planned[$group][$delta]['start_date'])) {
        unset($planned[$group][$delta]);
      }
    }

    // Process Links.
    if (!empty($setting['links'])) {
      $valueLinks = $entity->get($setting['links'])->getValue();
      foreach ($valueLinks as $valueLink) {
        $links[] = [
          'id' => implode('-', [$entity->id(), $valueLink['first']]),
          'source' => $entity->id(),
          'target' => (int) $valueLink['first'],
          'type' => $valueLink['second'],
          'lag' => $valueLink['third'] ?? NULL,
        ];
      }
    }

    // Get type item.
    $data[$group][$delta]['type_entity'] = $entity->getEntityTypeId();

    // Get bundle item.
    $data[$group][$delta]['bundle_entity'] = $entity->bundle();

    // Get parent_id item.
    if ($entity->getEntityTypeId() === "paragraph") {
      $data[$group][$delta]['parent_id_entity'] = $entity->get('parent_id')->value;
    }

    // Get parent_field_name item.
    if ($entity->getEntityTypeId() === "paragraph") {
      $data[$group][$delta]['parent_field_name_entity'] = $entity->get('parent_field_name')->value;
    }

    // Get parent_type item.
    if ($entity->getEntityTypeId() === "paragraph") {
      $data[$group][$delta]['parent_type_entity'] = $entity->get('parent_type')->value;
    }
  }

  // Fill empty start task.
  if (!empty($minDate) && !empty($emptyDate)) {
    $beginDate = min($minDate);
    foreach ($emptyDate as $dateFill) {
      $data[$dateFill['group']][$dateFill['delta']][$dateFill['field']] = date($setting['format_date'], $beginDate);
    }
  }
  if (!empty($maxDate)) {
    $endDate = date($setting['format_date'], max($maxDate));
  }

  if (count($data) == 1) {
    $data = current($data);
    if (!empty($planned)) {
      $planned = current($planned);
    }
  }
  else {
    $originData = $data;
    $data = [];
    foreach ($originData as $group => $tasks) {
      $groupName = $group;
      if (!empty($groupLabel[$group])) {
        $groupName = $groupLabel[$group];
      }
      elseif (!empty($entity_types[$group])) {
        $groupName = $entity_types[$group]['label'];
      }
      $data[] = [
        'id' => $group,
        'text' => $groupName,
        'type' => 'project',
        'open' => TRUE,
        'parent' => 0,
        'group_field' => $setting['group_field'],
        'type_entity' => reset($tasks)['type_entity'],
        'bundle_entity' => reset($tasks)['bundle_entity'],
        'parent_id_entity' => reset($tasks)['parent_id_entity'] ? reset($tasks)['parent_id_entity'] : "",
        'parent_type_entity' => reset($tasks)['parent_type_entity'] ? reset($tasks)['parent_type_entity'] : "",
        'parent_field_name_entity' => reset($tasks)['parent_field_name_entity'] ? reset($tasks)['parent_field_name_entity'] : "",
      ];
      foreach ($tasks as $delta => $task) {
        if (empty($task['parent'])) {
          $task['parent'] = $group;
        }
        $data[] = $task;
      }
    }

    if (!empty($planned)) {
      $originPlanned = $planned;
      $planned = [];
      foreach ($originPlanned as $group => $tasks) {
        $groupName = $group;
        if (!empty($groupLabel[$group])) {
          $groupName = $groupLabel[$group];
        }
        elseif (!empty($entity_types[$group])) {
          $groupName = $entity_types[$group]['label'];
        }
        $planned[] = [
          'id' => $group,
          'text' => $groupName,
          'type' => 'project',
          'open' => TRUE,
        ];
        foreach ($tasks as $delta => $task) {
          if (empty($task['parent'])) {
            $task['parent'] = $group;
          }
          $planned[] = $task;
        }
      }
    }
  }
  if (!empty($data)) {
    $outData['data'] = array_values($data);
    if (!empty($planned)) {
      $outData['data']['planned'] = array_values($planned);
    }
    if (!empty($links)) {
      $outData['links'] = $links;
    }
    $outData['marker']['start_date'] = date('Y-m-d', $beginDate ?? strtotime('now'));
    if (!empty($endDate)) {
      $outData['marker']['end_date'] = date('Y-m-d', $endDate);
    }
  }
  return $outData;
}
