/**
 * @file
 */

(function (Drupal, $, once) {
  Drupal.behaviors.gantt = {
    attach: (context, settings) => {
      $(once('gantt', '.showGantt', context)).each(function () {
        if (settings.gantt) {
          // Marker today.
          function markerToday(gantt) {
            let dateToStr = gantt.date.date_to_str(gantt.config.task_date);

            let id = gantt.addMarker({
              start_date: new Date(),
              css: "today",
              title: dateToStr(new Date())
            });
            setInterval(function () {
              let today = gantt.getMarker(id);
              if (today) {
                today.start_date = new Date();
                today.title = dateToStr(today.start_date);
                gantt.updateMarker(id);
              }
            }, 1000 * 60);
            // Make resize marker for two columns.
            gantt.attachEvent("onColumnResizeStart", function (ind, column) {
              if (!column.tree || ind == 0) {
                return;
              }

              setTimeout(function () {
                let marker = document.querySelector(".gantt_grid_resize_area");
                if (!marker) {
                  return;
                }
                let cols = gantt.getGridColumns();
                let delta = cols[ind - 1].width || 0;
                if (!delta) {
                  return;
                }

                marker.style.boxSizing = "content-box";
                marker.style.marginLeft = -delta + "px";
                marker.style.paddingRight = delta + "px";
              }, 1);
            });
          }

          // Undo Redo.
          function undo() {
            gantt.templates.scale_cell_class = function (date) {
              if (!gantt.isWorkTime(date)) {
                return "weekend";
              }
            };
            gantt.templates.timeline_cell_class = function (item, date) {
              if (!gantt.isWorkTime(date)) {
                return "weekend";
              }
            };
            gantt.attachEvent("onBeforeAutoSchedule", function () {
              gantt.message("Recalculating project schedule...");
              return true;
            });
            gantt.attachEvent("onAfterTaskAutoSchedule", function (task, new_date, constraint, predecessor) {
              gantt.message({
                text: "<b>" + task.text + "</b> has been rescheduled to " + gantt.templates.task_date(new_date) + " due to <b>" + predecessor.text + "</b> constraint",
                expire: 4000
              });
            });
          }

          // Expand at bottom.
          function expand(gantt) {
            gantt.attachEvent("onCollapse", function () {
              let icon = gantt.toggleIcon;
              if (icon) {
                icon.className = icon.className.replace("fullscreen-exit", "bi-arrows-fullscreen");
              }
            });
            gantt.attachEvent("onExpand", function () {
              let icon = gantt.toggleIcon;
              if (icon) {
                icon.className = icon.className.replace("bi-arrows-fullscreen", "fullscreen-exit");
              }
            });
            gantt.attachEvent("onTemplatesReady", function () {
              let toggle = document.createElement("i");
              toggle.className = "bi bi-arrows-fullscreen gantt-fullscreen";
              gantt.toggleIcon = toggle;
              gantt.$container.appendChild(toggle);
              toggle.onclick = function () {
                gantt.ext.fullscreen.toggle();
              };
            });
          }

          function urlGen(url, param = []) {
            let regex = /-arg_[0-9]-/g;
            let args = url.match(regex);
            args.forEach(function (element, index) {
              url = url.replace(element, param[index])
            })
            return url;
          }

          // User owner.
          function byId(list, id) {
            for (let i = 0; i < list.length; i++) {
              if (list[i].key == parseInt(id)) {
                return list[i].label || "";
              }
            }
            return "";
          }

          function owners(gantt, owner) {
            gantt.form_blocks["multiselect"] = {
              render: function (sns) {
                let height = (sns.height || "23") + "px";
                let html = "<div class='gantt_cal_ltext gantt_cal_chosen gantt_cal_multiselect' style='height:" + height + ";width: 100%'><select data-placeholder='…' class='chosen-select' multiple>";
                if (sns.options) {
                  for (var i = 0; i < sns.options.length; i++) {
                    if (sns.unassigned_value !== undefined && sns.options[i].key == sns.unassigned_value) {
                      continue;
                    }
                    html += "<option value='" + sns.options[i].key + "'>" + sns.options[i].label + "</option>";
                  }
                }
                html += "</select></div>";
                return html;
              },

              set_value: function (node, value, ev, sns) {
                node.style.overflow = "visible";
                node.parentNode.style.overflow = "visible";
                node.style.display = "inline-block";
                var select = $(node.firstChild);

                if (value) {
                  value = (value + "").split(",");
                  select.val(value);
                }
                else {
                  select.val([]);
                }
                if (typeof select.chosen === 'function') {
                  select.chosen();
                }
                if (sns.onchange) {
                  select.change(function () {
                    sns.onchange.call(this);
                  })
                }
                select.trigger('chosen:updated');
                select.trigger("change");
              },

              get_value: function (node, ev) {
                var value = $(node.firstChild).val();
                return value;
              },

              focus: function (node) {
                $(node.firstChild).focus();
              }
            };

            gantt.locale.labels.column_owner = Drupal.t("Owner");
            gantt.locale.labels.section_owner = Drupal.t("Owner");
            gantt.config.resource_property = "staff";
            gantt.serverList("staff", owner);

            gantt.attachEvent("onTaskCreated", function (task) {
              task.staff = [];
              task.calendar_id = null;
              return true;
            });
            // Check column owner is existing?
            let isColumnExist = false;
            gantt.config.columns.forEach(function (item) {
              if (item.hasOwnProperty("name")) {
                if (item.name === "owner") {
                  isColumnExist = true;
                }
                if (item.name === "text") {
                  item.width = 300;
                }
                if (item.name === "duration") {
                  item.resize = true;
                }
              }
            });
            if (!isColumnExist) {
              gantt.config.columns.unshift({
                name: "owner", width: 80, align: "center", template: (task) => {
                  if (task.type == gantt.config.types.project) {
                    return "";
                  }
                  var result = "";
                  let owners = task.owner;
                  if (!owners || !owners.length) {
                    let unassigned = Drupal.t("Unassigned");
                    return unassigned;
                  }
                  if (owners.length == 1) {
                    return byId(gantt.serverList('staff'), task.owner[0]);
                  }
                  owners.forEach((ownerId) => {
                    let owner = byId(gantt.serverList('staff'), ownerId);
                    if (!owner) {
                      return;
                    }
                    result += "<div class='owner-label' title='" + owner + "'>" + owner.charAt(0).toUpperCase() + "</div>";
                  });
                  return result;
                }, resize: true
              });

              gantt.config.lightbox.sections.push(
                {
                  name: "owner",
                  height: 35,
                  type: "multiselect",
                  options: gantt.serverList("staff"),
                  map_to: "owner"
                }
              );
            }

            gantt.attachEvent("onDataRender", function () {
              let styleId = "dynamicGanttStyles";
              let element = document.getElementById(styleId);
              if (!element) {
                element = document.createElement("style");
                element.id = styleId;
                document.querySelector("head").appendChild(element);
              }
              let html = [];
              let resources = gantt.serverList("staff");

              resources.forEach(function (r) {
                html.push(".gantt_task_line.gantt_resource_" + r.key + "{" +
                  "background-color:" + r.backgroundColor + "; " + "color:" + r.textColor + ";" + "}");
                html.push(".gantt_row.gantt_resource_" + r.key + " .gantt_cell:nth-child(1) .gantt_tree_content{" +
                  "background-color:" + r.backgroundColor + "; " + "color:" + r.textColor + ";" + "}");
              });
              element.innerHTML = html.join("");
            });

          }

          function resource(gantt, resource, label, id_resource, show_column = [], show_lightbox = []) {
            gantt.form_blocks["multiselect"] = {
              render: function (sns) {
                let height = (sns.height || "23") + "px";
                let html = "<div class='gantt_cal_ltext gantt_cal_chosen gantt_cal_multiselect' style='min-height:" + height + ";width: 100%'><select data-placeholder='…' class='chosen-select' multiple>";
                if (sns.options) {
                  for (var i = 0; i < sns.options.length; i++) {
                    if (sns.unassigned_value !== undefined && sns.options[i].key == sns.unassigned_value) {
                      continue;
                    }
                    html += "<option value='" + sns.options[i].key + "'>" + sns.options[i].label + "</option>";
                  }
                }
                html += "</select></div>";
                return html;
              },

              set_value: function (node, value, ev, sns) {
                node.style.overflow = "visible";
                node.parentNode.style.overflow = "visible";
                node.style.display = "inline-block";
                let select = $(node.firstChild);

                if (value) {
                  value = (value + "").split(",");
                  select.val(value);
                }
                else {
                  select.val([]);
                }

                select.chosen();
                if (sns.onchange) {
                  select.change(function () {
                    sns.onchange.call(this);
                  })
                }
                select.trigger('chosen:updated');
                select.trigger("change");
              },

              get_value: function (node, ev) {
                let value = $(node.firstChild).val();
                return value;
              },

              focus: function (node) {
                $(node.firstChild).focus();
              }
            };

            gantt.locale.labels["section_" + id_resource] = Drupal.t(label);
            gantt.locale.labels["column_" + id_resource] = Drupal.t(label);
            gantt.config.resource_property = id_resource;
            gantt.serverList(id_resource, resource);

            gantt.attachEvent("onTaskCreated", function (task) {
              task.staff = [];
              task.calendar_id = null;
              return true;
            });
            // Check column is existing?
            let colExists = gantt.config.columns.findIndex((element) => element.name == id_resource);

            // Show column.
            if (show_column.includes(id_resource) && colExists < 0) {
              gantt.config.columns.unshift({
                name: id_resource,
                width: 80,
                align: "center",
                template: (task) => {
                  if (task.type == gantt.config.types.project) {
                    return "";
                  }
                  let result = "";
                  let resources = task[id_resource];
                  if (!resources || !resources.length) {
                    let unassigned = Drupal.t("Unassigned");
                    return unassigned;
                  }
                  if (resources.length == 1) {
                    return byId(gantt.serverList(id_resource), task[id_resource][0]);
                  }
                  resources.forEach((resourceId) => {
                    let resource_name = byId(gantt.serverList(id_resource), resourceId);
                    if (!resource_name) {
                      return;
                    }
                    result += "<div class='owner-label' title='" + resource_name + "'>" + resource_name.charAt(0).toUpperCase() + "</div>";
                  });
                  return result;
                },
                resize: true
              });
            }

            // Show in light box.
            if (show_lightbox.includes(id_resource)) {
              gantt.config.lightbox.sections.push(
                {
                  name: id_resource,
                  height: 35,
                  type: "multiselect",
                  options: gantt.serverList(id_resource),
                  map_to: id_resource
                }
              );
            }

            gantt.attachEvent("onDataRender", function () {
              let styleId = "dynamicGanttStyles";
              let element = document.getElementById(styleId);
              if (!element) {
                element = document.createElement("style");
                element.id = styleId;
                document.querySelector("head").appendChild(element);
              }
              let html = [];
              let resources = gantt.serverList(id_resource);

              resources.forEach(function (r) {
                html.push(".gantt_task_line.gantt_resource_" + r.key + "{" +
                  "background-color:" + r.backgroundColor + "; " + "color:" + r.textColor + ";" + "}");
                html.push(".gantt_row.gantt_resource_" + r.key + " .gantt_cell:nth-child(1) .gantt_tree_content{" +
                  "background-color:" + r.backgroundColor + "; " + "color:" + r.textColor + ";" + "}");
              });
              element.innerHTML = html.join("");
            });

          }

          // Lightbox.
          function setLightBox(gantt, setting) {
            let formatter = gantt.ext.formatters.durationFormatter({
              enter: "day",
              store: "hour",
              format: "day",
              hoursPerDay: 24,
              hoursPerWeek: 40,
              daysPerMonth: 30,
              short: true
            });

            // Light box task.
            gantt.config.lightbox.sections = [
              {
                name: "description",
                height: 70,
                map_to: "text",
                type: "textarea",
                focus: true
              },
              {
                name: "type",
                height: "auto",
                type: "typeselect",
                map_to: "type",
                filter: function (name, value) {
                  return !!(value != gantt.config.types.project);
                }
              },
              {name: "time", type: "duration", map_to: "auto"}
            ];

            if (setting.select_parent) {
              let position = gantt.config.lightbox.project_sections.findIndex((element) => element.name == "type")
              gantt.config.lightbox.sections.splice(position + 1, 0, {
                name: "parent",
                type: "parent",
                allow_root: "true",
                root_label: "No parent",
                filter: function (id, task) {
                  return true;
                }
              })
            }

            if (setting.add_to_kanban) {
              let position = gantt.config.lightbox.sections.findIndex((element) => element.name == "time")
              gantt.config.lightbox.sections.splice(position + 1, 0, {
                name: "field_reference_task_kanban",
                type: "checkbox",
                map_to: "field_reference_task_kanban",
                options: [{"key": 1, "label": " Link task to kanban"}],
              })
            }

            if (!setting.use_cdn) {
              let position = gantt.config.lightbox.sections.findIndex((element) => element.name == "time")
              gantt.config.lightbox.sections.splice(position + 1, 0, {
                name: "baseline",
                button: true,
                type: "duration_optional",
                map_to: {start_date: "planned_start", end_date: "planned_end"}
              })
            }

            // Light box project.
            gantt.config.lightbox.project_sections = [
              {
                name: "description",
                height: 70,
                map_to: "text",
                type: "textarea",
                focus: true
              },
              {name: "time", type: "duration", readonly: true, map_to: "auto"}
            ];

            if (setting.select_parent) {
              let position = gantt.config.lightbox.project_sections.findIndex((element) => element.name == "type")
              gantt.config.lightbox.project_sections.splice(position + 1, 0, {
                name: "parent",
                type: "parent",
                allow_root: "true",
                root_label: "No parent",
                filter: function (id, task) {
                  return true;
                }
              })
            }

            if (setting.add_to_kanban) {
              let position = gantt.config.lightbox.project_sections.findIndex((element) => element.name == "time")
              gantt.config.lightbox.project_sections.splice(position + 1, 0, {
                name: "field_reference_task_kanban",
                type: "checkbox",
                map_to: "field_reference_task_kanban",
                options: [{"key": 1, "label": " Link task to kanban"}],
              })
            }

            if (!setting.use_cdn) {
              let position = gantt.config.lightbox.project_sections.findIndex((element) => element.name == "time")
              gantt.config.lightbox.project_sections.splice(position + 1, 0, {
                name: "baseline",
                type: "duration_optional",
                map_to: {start_date: "planned_start", end_date: "planned_end"}
              })
            }

            // Light box milestone.
            gantt.config.lightbox.milestone_sections = [
              {
                name: "description",
                height: 70,
                map_to: "text",
                type: "textarea",
                focus: true
              },
              {
                name: "type",
                height: "auto",
                type: "typeselect",
                map_to: "type",
                filter: function (name, value) {
                  return !!(value != gantt.config.types.project);
                }
              },
              {
                name: "parent",
                type: "parent",
                allow_root: "true",
                root_label: "No parent",
                filter: function (id, task) {
                  if (task.$level > 1) {
                    return false;
                  }
                  else {
                    return true;
                  }
                }
              },
              {
                name: "time",
                type: "duration",
                single_date: true,
                map_to: "auto",
                formatter: formatter
              }
            ];

            if (setting.add_to_kanban) {
              let position = gantt.config.lightbox.milestone_sections.findIndex((element) => element.name == "time")
              gantt.config.lightbox.milestone_sections.splice(position + 1, 0, {
                name: "field_reference_task_kanban",
                type: "checkbox",
                map_to: "field_reference_task_kanban",
                options: [{"key": 1, "label": " Link task to kanban"}],
              })
            }

          }

          // Tooltip.
          function setTooltip(gantt, setting) {
            // Set format date tooltips.
            gantt.templates.tooltip_date_format = function (date) {
              let formatFunc = gantt.date.date_to_str(setting.date_format);
              return formatFunc(date);
            };

            // Setup tooltip content.
            gantt.templates.tooltip_text = function (start, end, task) {
              let text_end = gantt.templates.tooltip_date_format(new Date(end.valueOf()));
              // Option Last of the day.
              if (setting.last_of_the_day) {
                text_end = gantt.templates.tooltip_date_format(new Date(end.valueOf() - 1));
              }
              let creator = Drupal.t("Unassigned");
              if (typeof task != 'undefined' && task.hasOwnProperty('creator_name')) {
                if (task.creator_name.length) {
                  creator = task.creator_name.join(", ")
                }
              }
              let html = "<b>" + Drupal.t('Task') + ":</b> " + task.text +
                "<br/><b>" + Drupal.t("Start") + ":</b> " + gantt.templates.tooltip_date_format(start) +
                "<br/><b>" + Drupal.t("End") + ":</b> " + text_end +
                "<br><b>" + Drupal.t("Duration") + ":</b> " + task.duration + Drupal.t(' days') +
                "<br><b>" + Drupal.t("Creator") + ":</b> " + creator;
              return html;
            };
          }

          function upload(file, callback, urlImport) {
            gantt.importFromMSProject({
              data: file,
              callback: function (project) {
                if (project) {
                  gantt.clearAll();
                  if (project.config.duration_unit) {
                    gantt.config.duration_unit = project.config.duration_unit;
                  }
                  // mark tasks import.
                  for (let i = 0; i < project.data.data.length; i++) {
                    project.data.data[i].isImport = true;
                  }
                  gantt.parse(project.data);
                  $.post(urlImport, project.data).done(function (data) {
                    console.log("Data imported: ", data);
                  });
                  if (callback) {
                    callback(project);
                  }
                }
              }
            });
          }

          function importMMP(ganttId, urlImport) {
            // Import microsoft project.
            let fileDnD = fileDragAndDrop();
            fileDnD.init(gantt.$container);

            function sendFile(file) {
              fileDnD.showUpload();
              upload(file, function () {
                fileDnD.hideOverlay();
              }, urlImport)
            }

            fileDnD.onDrop(sendFile);
            // Manual Upload file MPP.
            $("#mspImportBtn" + ganttId).click(function (event) {
              event.preventDefault();
              let fileInput = document.getElementById("mspFile" + ganttId);
              if (fileInput.files[0]) {
                sendFile(fileInput.files[0]);
              }
            });
          }

          function native_dialog(gantt, setting) {
            gantt.attachEvent("onTaskDblClick", function (id, e) {
              if (id == null) {
                return false;
              }
              let task = gantt.getTask(id);
              if (task['type'] == 'project') {
                return false;
              }
              else {
                let url = task.link_detail;
                if (!url.length) {
                    return false;
                }
                let ajaxSettings = {
                  url: url,
                  dialogType: 'modal',
                  dialog: {width: '80%'},
                };
                Drupal.ajax(ajaxSettings).execute();
                return false;
              }
            });

            gantt.attachEvent("onBeforeLightbox", function (id) {
              let task = gantt.getTask(id);
              if (task == null) {
                return false;
              }
              if (task.$new) {
                if (setting.add_task !== true) {
                  gantt.deleteTask(task.id);
                  return false;
                }
                let add_link = setting.add_link + '&parent=' + task.parent + '&parent_field=' + setting.parent_field;
                let ajaxSettings = {
                  url: add_link,
                  dialogType: 'modal',
                  dialog: {width: '80%'},
                };
                Drupal.ajax(ajaxSettings).execute();
                gantt.deleteTask(task.id);
                return false;
              }
              return true;
            });

          }

          let plugins = {
            fullscreen: true,
            marker: true,
            critical_path: true,
            tooltip: true,
            multiselect: true,
            auto_scheduling: true,
            drag_timeline: true,
            grouping: true,
            overlay: true,
            undo: true,
          };

          let i18n = {
            date: {
              month_full: [Drupal.t("January"), Drupal.t("February"), Drupal.t("March"), Drupal.t("April"), Drupal.t("May"), Drupal.t("June"),
                Drupal.t("July"), Drupal.t("August"), Drupal.t("September"), Drupal.t("October"), Drupal.t("November"), Drupal.t("December")],
              month_short: [Drupal.t("Jan"), Drupal.t("Feb"), Drupal.t("Mar"), Drupal.t("Apr"), Drupal.t("May"), Drupal.t("Jun"), Drupal.t("Jul"), Drupal.t("Aug"), Drupal.t("Sep"), Drupal.t("Oct"), Drupal.t("Nov"), Drupal.t("Dec")],
              day_full: [Drupal.t("Sunday"), Drupal.t("Monday"), Drupal.t("Tuesday"), Drupal.t("Wednesday"), Drupal.t("Thursday"), Drupal.t("Friday"), Drupal.t("Saturday")],
              day_short: [Drupal.t("Sun"), Drupal.t("Mon"), Drupal.t("Tue"), Drupal.t("Wed"), Drupal.t("Thu"), Drupal.t("Fri"), Drupal.t("Sat")]
            },
            labels: {
              new_task: Drupal.t("New task"),
              icon_save: Drupal.t("Save"),
              icon_cancel: Drupal.t("Cancel"),
              icon_details: Drupal.t("Details"),
              icon_edit: Drupal.t("Edit"),
              icon_delete: Drupal.t("Delete"),
              gantt_save_btn: Drupal.t("New Label"),
              gantt_cancel_btn: Drupal.t("New Label"),
              gantt_delete_btn: Drupal.t("New Label"),
              confirm_deleting: Drupal.t("Task will be deleted permanently, are you sure?"),
              section_description: Drupal.t("Description"),
              section_time: Drupal.t("Time period"),
              section_type: Drupal.t("Type"),
              section_baseline: Drupal.t("Planned"),
              baseline_enable_button: Drupal.t('Set'),
              baseline_disable_button: Drupal.t('Remove'),
              details_btn: Drupal.t("Details"),
              section_parent: Drupal.t("Parent task"),

              /* grid columns */
              column_wbs: "WBS",
              column_text: Drupal.t("Task name"),
              column_start_date: Drupal.t("Start time"),
              column_duration: Drupal.t("Duration"),

              /* link confirmation */
              link: Drupal.t("Link"),
              confirm_link_deleting: Drupal.t("will be deleted"),
              link_start: Drupal.t(" (start)"),
              link_end: Drupal.t(" (end)"),

              type_task: Drupal.t("Task"),
              type_project: Drupal.t("Project"),
              type_milestone: Drupal.t("Milestone"),
              type_placeholder: Drupal.t("Placeholder"),

              minutes: Drupal.t("Minutes"),
              hours: Drupal.t("Hours"),
              days: Drupal.t("Days"),
              weeks: Drupal.t("Week"),
              months: Drupal.t("Months"),
              years: Drupal.t("Years"),

              /* message popup */
              message_ok: Drupal.t("OK"),
              message_cancel: Drupal.t("Cancel"),

              /* constraints */
              section_constraint: Drupal.t("Constraint"),
              constraint_type: Drupal.t("Constraint type"),
              constraint_date: Drupal.t("Constraint date"),
              asap: Drupal.t("As Soon As Possible"),
              alap: Drupal.t("As Late As Possible"),
              snet: Drupal.t("Start No Earlier Than"),
              snlt: Drupal.t("Start No Later Than"),
              fnet: Drupal.t("Finish No Earlier Than"),
              fnlt: Drupal.t("Finish No Later Than"),
              mso: Drupal.t("Must Start On"),
              mfo: Drupal.t("Must Finish On"),

              /* resource control */
              resources_filter_placeholder: Drupal.t("type to filter"),
              resources_filter_label: Drupal.t("hide empty")
            }
          };
          $(document).ready(function () {
            $.each(settings.gantt, function (key, ganttField) {

              gantt.plugins(plugins);
              markerToday(gantt);
              expand(gantt);
              setLightBox(gantt, ganttField);
              setTooltip(gantt, ganttField);

              gantt.config.auto_types = true;
              gantt.config.order_branch = "marker";
              gantt.config.order_branch_free = true;
              gantt.config.grid_resize = true;

              gantt.config.sort = true;

              gantt.config.fit_tasks = true;
              gantt.$zoomToFit = false;
              gantt.config.reorder_grid_columns = true;
              gantt.config.open_tree_initially = true;

              gantt.i18n.setLocale(i18n);
              gantt.config.xml_date = ganttField.date_format;
              gantt.config.date_format = ganttField.date_format;
              gantt.config.date_grid = ganttField.date_format;

              // Ajax link action.
              let configAjax = {
                url: ganttField.ajax,
                mode: "POST"
              };

              // Css class.
              gantt.templates.grid_row_class = function (start, end, task) {
                let css = [];
                if (task.$virtual || task.type === gantt.config.types.project) {
                  css.push("summary-row");
                }
                if (task.progress >= 1) {
                  css.push("summary-grid-row-green");
                }
                if (task.owner) {
                  css.push("gantt_resource_task gantt_resource_" + task.owner);
                }

                return css.join(" ");
              };

              gantt.templates.task_row_class = function (start, end, task) {
                let css = [];
                if (task.$virtual || task.type === gantt.config.types.project) {
                  css.push("summary-row");
                }
                if (task.progress >= 1) {
                  css.push("summary-task-row-green");
                }
                if (task.owner) {
                  css.push("gantt_resource_task gantt_resource_" + task.owner);
                }

                return css.join(" ");
              };

              gantt.templates.task_class = function (start, end, task) {
                let css = [];
                if (task.$virtual || task.type === gantt.config.types.project) {
                  css.push("summary-bar");
                }
                /* Add class task when exist base line. */
                if (ganttField.has_planned && !ganttField.split_actual_planned && !ganttField.use_cdn) {
                  css.push("exist_base_line");

                  /* Add class Overdue. */
                  if (task.planned_end) {
                    css.push("has-baseline");
                    if (end.getTime() > task.planned_end.getTime()) {
                      css.push('overdue');
                    }
                  }
                }
                return css.join(" ");
              };

              // Drag timeline.
              gantt.config.drag_timeline = {
                ignore: ".gantt_task_line, .gantt_task_link",
                useKey: "ctrlKey"
              };

              gantt.attachEvent("onBeforeTaskDrag", function (id, mode, e) {
                if (e.ctrlKey) {
                  return false;
                }
                return true;
              });

              // Set width text column.
              let colText = gantt.config.columns.findIndex((element) => element.name == "text");
              if (colText >= 0) {
                gantt.config.columns[colText].min_width = 200;
              }

              // Set resize duration column.
              let colDuration = gantt.config.columns.findIndex((element) => element.name == "duration");
              if (colDuration >= 0) {
                gantt.config.columns[colDuration].resize = true;
              }

              // Custom button.
              if (ganttField.show_button_detail) {
                gantt.attachEvent("onGanttReady", function (task) {
                  gantt.config.buttons_left = ["gantt_save_btn", "details_btn", "gantt_cancel_btn"];
                  gantt.config.buttons_right = ["gantt_delete_btn"];
                });

                gantt.attachEvent("onLightboxButton", function (button_id, node, e, task) {
                  if (button_id == "details_btn") {
                    let id = gantt.getState().lightbox;
                    let task = gantt.getTask(id);
                    if (!task.$new) {
                      let url = task.link_detail;
                      if(url == undefined) {
                        url = ganttField.link_detail.edit;
                        if(task.hasOwnProperty("readonly")) {
                          url = ganttField.link_detail.display;
                        }
                        url = urlGen(url, [id]);
                      }

                      let ajaxSettings = {
                        url: url,
                        dialogType: 'modal',
                        dialog: {width: '80%'},
                      };
                      Drupal.ajax(ajaxSettings).execute();
                      gantt.hideLightbox();
                      return false;
                    }
                  }
                });
              }

              // Button action.
              gantt.attachEvent("onLightboxSave", function (id, task, is_new) {
                if (gantt.isReadonly(id)) {
                  gantt.alert(Drupal.t("You cannot edit this task."));
                  return false;
                }
                if (task.text.length > 255) {
                  gantt.alert(Drupal.t("Description cannot be too long"));
                  return false;
                }
                return true;
              })

              gantt.attachEvent("onLightboxDelete", function (id, task, is_new) {
                if (gantt.isReadonly(id)) {
                  gantt.alert(Drupal.t("You cannot delete this task."));
                  return false;
                }
                return true;
              })

              // Show progress %.
              if (ganttField.has_duration) {
                gantt.templates.progress_text = function (start, end, task) {
                  return "<span style='text-align:left;'>" + Math.round(task.progress * 100) + "% </span>";
                };
              }

              // Add column +.
              if (!ganttField.add_task) {
                $.each(gantt.config.columns, function (index, col) {
                  if (col.name === 'add') {
                    delete col.name;
                  }
                });
              }

              // Change mode view readonly.
              if (ganttField.edit_task === false) {
                gantt.config.readonly = true;
              }

              // Switch dialog native drupal.
              if (ganttField.native_dialog) {
                native_dialog(gantt, ganttField);
              }

              // Add custom column custom field.
              if (ganttField.custom_field) {
                let customColumnExist = gantt.config.columns.findIndex((element) => element.name == ganttField.custom_field);
                if (customColumnExist < 0) {
                  gantt.config.columns.splice(gantt.config.columns.length - 1, 0, {
                    name: ganttField.custom_field,
                    label: ganttField.custom_field_name,
                    align: "center",
                    resize: true
                  });
                  gantt.locale.labels['section_' + ganttField.custom_field] = ganttField.custom_field_name;
                }
                let customLightBoxExist = gantt.config.lightbox.sections.findIndex((element) => element.name == ganttField.custom_field);
                if (customLightBoxExist < 0) {
                  gantt.config.lightbox.sections.push(
                    {
                      name: ganttField.custom_field,
                      type: "number",
                      map_to: ganttField.custom_field,
                    }
                  );
                  gantt.form_blocks["number"] = {
                    render: function (sns) {
                      return '<input class="form-control"  type="number" min="0">';
                    },
                    set_value: function (node, value, task) {
                      node.value = value;
                    },
                    get_value: function (node, task) {
                      return node.value || task.options;
                    },
                    focus: function (node) {
                    }
                  };
                }
              }

              // Add baseline on row.
              // Show 2nd chart for actual date or baseline.
              if (ganttField.has_planned && !ganttField.split_actual_planned && !ganttField.use_cdn) {
                gantt.config.bar_height = 16;
                gantt.config.row_height = 40;

                /* Adding baseline display. */
                if (gantt.hasOwnProperty('addTaskLayer') && !ganttField.use_cdn) {
                  gantt.addTaskLayer({
                    renderer: {
                      render: function draw_planned(task) {
                        /* Add class link when exist base line. */
                        gantt.templates.link_class = function (link) {
                          return "exist_base_line_link";
                        };
                        /* Add base line. */
                        if (task.planned_start && task.planned_end) {
                          var sizes = gantt.getTaskPosition(task, task.planned_start, task.planned_end);
                          var el = document.createElement('div');
                          el.className = 'baseline';
                          el.style.left = sizes.left + 'px';
                          el.style.width = sizes.width + 'px';
                          el.style.top = sizes.top + gantt.config.bar_height + 13 + 'px';
                          return el;
                        }
                        return false;
                      },
                      /* Define getRectangle in order to hook layer with the. */
                      getRectangle: function (task, view) {
                        if (task.planned_start && task.planned_end) {
                          return gantt.getTaskPosition(task, task.planned_start, task.planned_end);
                        }
                        return null;
                      }
                    }
                  });

                  /* Calculator Overdue. */
                  gantt.templates.rightside_text = function (start, end, task) {
                    if (task.planned_end) {
                      if (end.getTime() > task.planned_end.getTime()) {
                        let overdue = Math.ceil(Math.abs((end.getTime() - task.planned_end.getTime()) / (24 * 60 * 60 * 1000)));
                        let text = `<b>${Drupal.t('Overdue')}: ${overdue} ${Drupal.t('days')}</b>`;
                        return text;
                      }
                    }
                  };

                  gantt.attachEvent("onTaskLoading", function (task) {
                    task.planned_start = gantt.date.parseDate(task.planned_start, "xml_date");
                    task.planned_end = gantt.date.parseDate(task.planned_end, "xml_date");
                    return true;
                  });
                }
              }

              // Add workTime and holiday.
              if (ganttField.work_time) {
                gantt.config.work_time = true;
                /* Add holiday. */
                if (ganttField.holidays) {
                  let holidays = ganttField.holidays.map(dateString => {
                    let dateParts = dateString.split('-');
                    let year = parseInt(dateParts[0]);
                    let month = parseInt(dateParts[1]) - 1;
                    let day = parseInt(dateParts[2]);
                    gantt.setWorkTime({
                      date: new Date(year, month, day),
                      hours: false
                    });
                    return new Date(year, month, day);
                  });
                }

                gantt.setWorkTime({day: 0, hours: true});
                gantt.setWorkTime({day: 6, hours: true});
                if (ganttField.work_day.length) {
                  const workDays = ganttField.work_day.map(day => {
                    gantt.setWorkTime({day: day, hours: false});
                    return day;
                  });
                }
                gantt.templates.timeline_cell_class = function (task, date) {
                  if (!gantt.isWorkTime({task: task, date: date})) {
                    return "weekend";
                  }
                  return "";
                };
              }

              // Set zoom config for charts.
              let zoomConfig = {
                minColumnWidth: 50,
                maxColumnWidth: 150,
                levels: [
                  {
                    name: "year",
                    scales: [
                      {unit: "year", format: "%Y", step: 1},
                    ]
                  },
                  {
                    name: "quarter",
                    scales: [
                      {unit: "year", format: "%Y", step: 1},
                      {
                        unit: "quarter", step: 1, format: function (date) {
                          let dateToStr = gantt.date.date_to_str("%M");
                          let endDate = gantt.date.add(gantt.date.add(date, 3, "month"), -1, "day");
                          return dateToStr(date) + " - " + dateToStr(endDate);
                        }
                      }
                    ]
                  },
                  {
                    name: "month",
                    scales: [
                      {unit: "year", format: "%Y", step: 1},
                      {unit: "month", format: "%M", step: 1}
                    ]
                  },
                  {
                    name: "week",
                    scales: [
                      {unit: "month", format: "%M %Y", step: 1},
                      {
                        unit: "week", step: 1, format: function (date) {
                          let dateToStr = gantt.date.date_to_str("%d/%m");
                          let endDate = gantt.date.add(gantt.date.add(date, 7, "day"), -1, "day");
                          let weekNum = gantt.date.date_to_str("%W")(date);
                          return "#" + weekNum + ", " + dateToStr(date) + " - " + dateToStr(endDate);
                        }
                      }
                    ]
                  },
                  {
                    name: "day",
                    scales: [
                      {unit: "month", format: "%M %Y", step: 1},
                      {unit: "day", format: "%d %M", step: 1}
                    ]
                  }
                ],
                useKey: "ctrlKey",
                trigger: "wheel",
                element: function () {
                  return gantt.$root.querySelector(".gantt_task");
                }
              }

              gantt.ext.zoom.init(zoomConfig);
              gantt.ext.zoom.setLevel("day");
              gantt.ext.zoom.attachEvent("onAfterZoom", function (level, config) {
                gantt.$root.closest(".gantt-wrapper").querySelector("[onchange^='set_scale']").value = config.name;
              })

              // Add show column end date.
              if (ganttField.show_end) {
                let indexCol = gantt.config.columns.findIndex((item) => item.name == 'start_date');
                gantt.config.columns.splice(indexCol + 1, 0, {
                  name: "end_date",
                  width: 80,
                  label: Drupal.t("End"),
                  align: "center",
                  resize: true,
                  template: function (task) {
                    return gantt.templates.date_grid(task.end_date)
                  }
                });
              }

              // Owner.
              if (ganttField.owner) {
                owners(gantt, ownerGantt[ganttField.id]);
              }

              // Set resource.
              if (ganttField.custom_resource.resource.length) {
                ganttField.custom_resource.resource.map(function (id_resource, index) {
                  resource(gantt, ganttField['custom_resource']['resource_server_list'][id_resource], ganttField['custom_resource']['resource_label'][id_resource], id_resource, ganttField['custom_resource']['resource_column'], ganttField['custom_resource']['resource_lightbox']);
                })
              }

              // Only links.
              gantt.attachEvent("onBeforeLinkAdd", function (id,link) {
                let taskSource = gantt.getTask(link.source);
                let taskTarget = gantt.getTask(link.target);
                if (gantt.isReadonly(link.target)) {
                  gantt.message({
                    type: "error",
                    text: Drupal.t("Can not link. <b>" + taskTarget.text + "</b> not permission."),
                    expire: 4000
                  });
                  return false;
                }
                if(taskSource.$source.includes(link.source + "-" + link.target) || taskTarget.$source.includes(link.target + "-" + link.source)) {
                  gantt.message({
                    type: "error",
                    text: Drupal.t("Link <b>" + taskSource.text + "</b> to <b>" + taskTarget.text + "</b> already exist."),
                    expire: 4000
                  });
                  return false;
                }
                return true;
              });

              // Automatic scheduling.
              if (ganttField.auto_schedule && !ganttField.use_cdn) {
                linkLagEditor(gantt);

                gantt.config.auto_scheduling = true;
                gantt.attachEvent("onAfterTaskAutoSchedule", function (task, new_date, constraint, predecessor) {
                  if (task && predecessor) {
                    gantt.message({
                      text: Drupal.t("<b>" + task.text + "</b> has been rescheduled to " + gantt.templates.task_date(new_date) + " due to <b>" + predecessor.text + "</b> constraint"),
                      expire: 4000
                    });
                  }
                });
              }

              // Display task search text.
              gantt.attachEvent("onBeforeTaskDisplay", function (id, task) {
                gantt.config.open_tree_initially = true;
                if (compare_input(gantt, key, id)) {
                  return true;
                }
                return false;
              });

              if ($('#' + ganttField.id).length) {
                let dataParagraph = $('#' + ganttField.id).data();
                configAjax.url += '?' + $.param(dataParagraph);
                ganttField.import += '?' + $.param(dataParagraph);
              }

              if (markerGantt[ganttField.id] !== undefined) {
                let dateToStr = gantt.date.date_to_str(gantt.config.task_date);
                let start = new Date(markerGantt[ganttField.id].start_date);
                gantt.addMarker({
                  start_date: start,
                  css: "status_line",
                  text: Drupal.t("Start project"),
                  title: Drupal.t("Start project") + ": " + dateToStr(start)
                });
                if (markerGantt[ganttField.id].end_date) {
                  let end = new Date(markerGantt[ganttField.id].end_date);
                  gantt.addMarker({
                    start_date: end,
                    css: "status_line",
                    text: Drupal.t("End project"),
                    title: Drupal.t("End project") + ": " + dateToStr(end)
                  });
                }
              }

              // Format end date.
              gantt.templates.task_end_date = function (date) {
                // Option Last of the day.
                if (ganttField.last_of_the_day) {
                  return gantt.templates.task_date(new Date(date.valueOf() - 1));
                }
                return gantt.templates.task_date(new Date(date.valueOf()));
              };
              let gridDateToStr = gantt.date.date_to_str(ganttField.date_format);
              gantt.templates.grid_date_format = function (date, column) {
                if (column === "end_date") {
                  // Option Last of the day.
                  if (ganttField.last_of_the_day) {
                    return gridDateToStr(new Date(date.valueOf() - 1));
                  }
                  return gridDateToStr(new Date(date.valueOf()));
                }
                else {
                  return gridDateToStr(date);
                }
              }

              // onMove task.
              gantt.attachEvent("onBeforeRowDragEnd", function (id, parent, tindex) {
                if (ganttField.group_field.length) {
                  if (id && parent) {
                    return false;
                  }
                  let task = gantt.getTask(id);
                  let task_target = gantt.getTask(parent);
                  let parents = [];
                  parents.push(task.id);
                  if (task.parent) {
                    gantt.eachParent(function (parent) {
                      parents.push(parent.id);
                    }, task.id)
                  }
                  let parents_target = [];
                  parents_target.push(task_target.id);
                  if (task_target.parent) {
                    gantt.eachParent(function (parent) {
                      parents_target.push(parent.id);
                    }, task_target.id)
                  }
                  let parent_root = parents[parents.length - 1];
                  let target_parent_root = parents_target[parents_target.length - 1];
                  if (parent_root != target_parent_root) {
                    gantt.message({type: "error", text: "Can not move " + task.text + " to " + task_target.text, expire: 4000})
                    return false;
                  }
                }
                return true;
              });

              // Error ajax.
              gantt.attachEvent("onAjaxError", function (request) {
                gantt.message({type: "error", text:"Error " + request.status + "!", expire: 4000})
                return true;
              });

              // Trigger gantt for mono chart.
              gantt.createDataProcessor(configAjax).attachEvent("onBeforeUpdate", function (id, state, data) {
                if (data.parent) {
                  let parentTask = gantt.getTask(data.parent);
                  if (parentTask.hasOwnProperty("group_field") && (parentTask.group_field === 'parent_field_name' || parentTask.group_field === 'parent_id')) {
                    data.group_field = parentTask.group_field;
                  }
                  if (parentTask.hasOwnProperty('type_entity')) {
                    data.type_entity = parentTask.type_entity;
                  }
                  if (parentTask.hasOwnProperty('bundle_entity')) {
                    data.bundle_entity = parentTask.bundle_entity;
                  }
                  if (parentTask.hasOwnProperty('parent_id_entity')) {
                    data.parent_id_entity = parentTask.parent_id_entity;
                  }
                  if (parentTask.hasOwnProperty('parent_type_entity')) {
                    data.parent_type_entity = parentTask.parent_type_entity;
                  }
                  if (parentTask.hasOwnProperty('parent_field_name_entity')) {
                    data.parent_field_name_entity = parentTask.parent_field_name_entity;
                  }
                }
                return true;
              });

              if ($('#' + ganttField.id).length) {
                gantt.init(ganttField.id);
              }
              if (dataGantt[ganttField.id] && dataGantt[ganttField.id].data !== undefined) {
                gantt.parse(dataGantt[ganttField.id]);
              }
            });
          });
        }
      });
    },
  };
}(Drupal, jQuery, once));
