<?php

namespace Drupal\gantt\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\views\Views;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Returns responses for Gantt routes.
 */
class GanttController extends ControllerBase implements ContainerInjectionInterface {


  /**
   * Query POST.
   *
   * @var array
   */
  private $post = [];

  /**
   * Query GET.
   *
   * @var array
   */
  private $get = [];

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The request Stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  private $requestStack;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * {@inheritDoc}
   */
  public function __construct(RouteMatchInterface $route_match, RequestStack $request_stack, EntityFieldManagerInterface $entityFieldManager) {
    $request = $request_stack->getCurrentRequest();
    $this->post = $request->request->all();
    $this->get = $request->query->all();
    $this->requestStack = $request_stack;
    $this->entityTypeManager = $this->entityTypeManager();
    $this->entityFieldManager = $entityFieldManager;
    $this->routeMatch = $route_match;
    $this->currentUser = $this->currentUser();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('current_route_match'),
      $container->get('request_stack'),
      $container->get('entity_field.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function loadView($view_id, $display_id, &$fieldSetting = []) {
    $view = Views::getView($view_id);
    $view->setDisplay($display_id);
    $view->initDisplay();
    $view->initHandlers();
    $fieldSetting = $view->getStyle()->options;
    $entity_type = $view->getBaseEntityType()->id();
    $entity_bundles = [];
    $filter = $view->filter;
    if (!empty($filter['type'])) {
      $entity_bundles = key($filter['type']->value);
    }
    $fieldSetting['view_type_entity'] = $entity_type;
    $fieldSetting['view_bundle_entity'] = $entity_bundles;
    // Get handlers setting formatter date in view.
    $fieldDate = $view->getHandlers('field')[$fieldSetting['start_date']];

    // Get field type of field links.
    $field_options = $view->display_handler->getOption('fields')[$fieldSetting['links']] ?? NULL;

    if ($field_options && $entity_type && $entity_bundles) {
      $field_definition = $this->entityFieldManager->getFieldDefinitions($entity_type, $entity_bundles)[$fieldSetting['links']] ?? NULL;
      if ($field_definition) {
        $field_type_links = $field_definition->getType();
      }
    }

    $dateType = $this->entityTypeManager()
      ->getStorage('date_format')
      ->load($fieldDate["settings"]["format_type"]);
    $patterns = !empty($dateType) ? explode(' ', $dateType->getPattern()) : ['d-m-Y'];
    $date_format = current(array_filter(
      $patterns,
      function ($value) {
        return strlen($value) >= 4;
      }
    ));
    $fieldSetting['format_date'] = $date_format;
    $fieldSetting['field_type_links'] = $field_type_links ?? NULL;
    return $this->entityTypeManager()->getStorage($entity_type);
  }

  /**
   * Add new / Edit / Delete Paragraphs.
   */
  public function ajax($view_id, $display_id) {
    $fieldSetting = [];
    $entityType = $this->loadView($view_id, $display_id, $fieldSetting);
    $bundle = !empty($fieldSetting['bundle']) ? $fieldSetting['bundle'] : 'gantt';
    $action = !empty($this->post["!nativeeditor_status"]) ? $this->post["!nativeeditor_status"] : FALSE;
    $output = [];
    if (!$action) {
      return new JsonResponse($output);
    }
    if ($this->get['gantt_mode'] == 'tasks') {
      $output = $this->actionTask($action, $entityType, $bundle, $fieldSetting);
    }
    if ($this->get['gantt_mode'] == 'links' && !empty($fieldSetting["links"])) {
      $output = $this->actionLinks($action, $entityType, $fieldSetting);
    }
    return new JsonResponse($output);
  }

  /**
   * Import file MPP.
   */
  public function import($view_id, $display_id) {
    $fieldSetting = [];
    $entityType = $this->loadView($view_id, $display_id, $fieldSetting);
    $bundle = !empty($fieldSetting['bundle']) ? $fieldSetting['bundle'] : 'gantt';
    $action = 'inserted';
    $output = ['action' => $action];
    $mapping = [];
    if (!empty($this->post["data"])) {
      $rawData = $this->post["data"];
      $rawLinks = $this->post["links"];
      $this->post = [];
      foreach ($rawData as $data) {
        $ganttData[$data['id']] =
        $this->post = [
          'text' => $data['text'],
          'duration' => $data['duration'],
          'progress' => $data['progress'],
          'open' => $data['open'],
          'parent' => $data['parent'],
          'type' => !empty($data['type']) ? $data['type'] : 'task',
          'start_date' => $data['$raw']["Start"],
          'end_date' => $data['$raw']["Finish"],
        ];
        if (!empty($mapping[$data['parent']])) {
          $this->post['parent'] = $mapping[$data['parent']];
        }
        $output = $this->actionTask($action, $entityType, $bundle, $fieldSetting);
        $mapping[$data['id']] = !empty($output['tid']) ? $output['tid'] : FALSE;
      }

      foreach ($rawLinks as $data) {
        $this->post = [
          'source' => $mapping[$data['source']],
          'target' => $mapping[$data['target']],
          'type' => $data['type'],
        ];
        $output = $this->actionLinks($action, $entityType, $fieldSetting);
      }
    }

    return new JsonResponse($output);
  }

  /**
   * Convert date return with format Y-m-d.
   */
  protected function formatDate($date, $type = 'date', $format = 'Y-m-d H:i:s') {
    $dateFormatStorage = $this->entityTypeManager->getStorage('date_format');
    $drupalShortFormat = $dateFormatStorage->load('short')->getPattern();
    if ($type == 'date') {
      $format = current(explode(' ', $format));
      $drupalShortFormat = current(explode(' ', $drupalShortFormat));
    }
    $convertDateTime = \DateTime::createFromFormat($drupalShortFormat, $date);
    if (!$convertDateTime) {
      $date = str_replace('/', '-', $date);
      $timestamp = strtotime($date);
      return date($format, $timestamp);
    }
    return $convertDateTime->format($format);
  }

  /**
   * Process Task.
   *
   * {@inheritDoc}
   */
  private function actionTask($action, $entityType, $bundle, $field_settings) {
    $output = ['action' => $action];
    if (!empty($this->post['isImport'])) {
      return new JsonResponse($output);
    }
    return $this->actionEntityTask($action, $entityType, $field_settings);
  }

  /**
   * Process Paragraphs Task.
   *
   * {@inheritDoc}
   */
  protected function actionEntityTask($action, $entityType, $field_settings) {
    if (empty($field_settings['format_date'])) {
      $field_settings['format_date'] = "Y-m-d";
    }
    $output = ['action' => $action];
    $parent_field = '';
    $parent_type = '';
    $bundle_entity = '';
    $entity_parent = NULL;
    $entity = NULL;
    if (!empty($this->post["id"]) && !empty($this->post['type_entity']) && $action != "inserted") {
      $entity = $this->entityTypeManager->getStorage($this->post['type_entity'])
        ->load($this->post["id"]);
    }
    // Check parent.
    if (empty($entity) && !empty($this->post["parent"]) && $this->post['parent'] != $this->post['parent_id_entity']) {
      $entity = $this->entityTypeManager->getStorage($this->post['type_entity'])
        ->load($this->post["parent"]);
    }
    if (empty($entity) && !empty($this->post["parent_id_entity"]) && !empty($this->post["parent_type_entity"]) && !empty($this->post["bundle_entity"])) {
      if (!empty($field_settings['group_field']) && !empty($field_settings['view_bundle_entity']) && $field_settings['view_type_entity'] == "paragraph") {
        if (!empty($this->post['parent_type_entity'])) {
          $parent_type = $this->post['parent_type_entity'];
        }
        if (!empty($this->post['parent_field_name_entity'])) {
          $parent_field = $this->post['parent_field_name_entity'];
        }
        if ($parent_type != '' && !empty($this->post['parent_id_entity'])) {
          $entity_parent = $this->entityTypeManager->getStorage($parent_type)
            ->load($this->post['parent_id_entity']);
        }
      }
      if (!empty($field_settings['view_bundle_entity']) && !empty($field_settings['view_type_entity'])) {
        $type_entity = $field_settings['view_type_entity'];
        $bundle_entity = $field_settings['view_bundle_entity'];
      }
    }
    if (!empty($entity_parent) && !empty($parent_type)) {
      $storage = $this->entityTypeManager->getStorage($parent_type);
      $storage->resetCache([$entity_parent->id()]);
    }
    if (empty($entity) && empty($field_settings['group_field']) && $action == 'inserted') {
      $entity = $entityType->create([
        'type' => $field_settings['view_bundle_entity'],
        'title' => $this->t("Task gantt"),
        'uid' => $this->currentUser->id(),
      ]);
    }
    if (!empty($entity)) {
      $bundle_entity = $entity->bundle();
      $type_entity = $entity->getEntityTypeId();
      if ($entity->getEntityTypeId() == "paragraph") {
        $entity_parent = $entity->getParentEntity();
      }
      if ($entity->getEntityTypeId() == "paragraph") {
        $parent_type = $entity->get("parent_type")->value;
      }
      if ($entity->getEntityTypeId() == "paragraph") {
        $parent_field = $entity->get("parent_field_name")->value;
      }
    }
    if ($action == 'inserted' && !empty($type_entity) && !empty($bundle_entity)) {
      $entity = $this->entityTypeManager->getStorage($type_entity)
        ->create(['type' => $bundle_entity]);
    }
    if (!empty($entity_parent) && !empty($parent_type)) {
      $storage = $this->entityTypeManager->getStorage($parent_type);
      $storage->resetCache([$entity_parent->id()]);
    }
    if (!empty($entity)) {
      if ($action == 'deleted') {
        if (!empty($entity_parent) && !empty($parent_field)) {
          $getField = $entity_parent->get($parent_field);
          $array_of_referenced_items = $getField->getValue();
          $index_to_remove = array_search($this->post['id'], array_column($array_of_referenced_items, 'target_id'));
          $getField->removeItem($index_to_remove);
          $entity_parent->save();
        }
        $entity->delete();
        return new JsonResponse($output);
      }

      $fieldDefinitions = $entity->getFieldDefinitions();
      $fieldType = $dateType = 'date';
      if (!empty($fieldDefinitions[$field_settings['start_date']])) {
        $dateSettings = $fieldDefinitions[$field_settings['start_date']]->getSettings();
        $dateType = $dateSettings["datetime_type"];
        $fieldType = $fieldDefinitions[$field_settings['start_date']]->getType();
      }

      // Set start date.
      $actual_date = FALSE;
      if (!empty($this->post['start_date']) && $this->post['start_date'] != "undefined") {
        $start_date = DrupalDateTime::createFromFormat($field_settings['format_date'], $this->post['start_date']);
        $end_date = DrupalDateTime::createFromFormat($field_settings['format_date'], $this->post['end_date']);

        // Option Last of the day.
        if ($field_settings['last_of_the_day']) {
          $end_date->modify("-1 day");
        }

        if ($fieldType == 'daterange' && !$start_date) {
          $actual_date = [
            'value' => $start_date->format("Y-m-d"),
            'end_value' => $end_date->format("Y-m-d"),
          ];
        }
      }
      // Set plan date.
      $planned_date = [
        'value' => NULL,
        'end_value' => NULL,
      ];
      if (!empty($this->post['planned_start']) && $this->post['planned_start'] != "undefined") {
        $planned_start = DrupalDateTime::createFromFormat($field_settings['format_date'], $this->post['planned_start']);
        $planned_end = DrupalDateTime::createFromFormat($field_settings['format_date'], $this->post['planned_end']);

        // Option Last of the day.
        if ($field_settings['last_of_the_day']) {
          $planned_end->modify("-1 day");
        }

        if ($fieldType == 'daterange' && $planned_start->getTimestamp()) {
          $planned_date = [
            'value' => $planned_start->format("Y-m-d"),
            'end_value' => $planned_end->format("Y-m-d"),
          ];
        }
      }
      // Set actual date and plan date.
      if ($actual_date) {
        $isPlanned = !empty($field_settings["baseline"]["planned_date"]) && !empty($field_settings['baseline']['planned_duration']);
        if (empty($isPlanned)) {
          $entity->set($field_settings['start_date'], $actual_date);
          $entity->set($field_settings['duration'], $this->post['duration']);
        }
        else {
          $entity->set($field_settings['baseline']['planned_date'], $actual_date);
          $entity->set($field_settings['baseline']['planned_duration'], $this->post['duration']);
        }
        if (!empty($planned_start) && !empty($planned_end) && !empty($field_settings['baseline']['planned_date'])) {
          $entity->set($field_settings['baseline']['planned_date'], $planned_date);
          $planned_duration = round(abs($planned_start->getTimestamp() - $planned_end->getTimestamp()) / 86400);
          $entity->set($field_settings['baseline']['planned_duration'], $planned_duration);
        }
      }
      // Set text.
      if (!empty($field_settings['text'])) {
        $entity->set($field_settings['text'], $this->post['text']);
      }
      // Set type.
      if (!empty($field_settings["type"]) && !empty($this->post["type"])) {
        $entity->set($field_settings['type'], $this->post["type"]);
      }
      // Set progress.
      if (!empty($field_settings['progress']) && !empty($this->post['progress'])) {
        $entity->set($field_settings['progress'], $this->post['progress']);
      }
      // Set parent.
      $parent = !empty($this->post['parent']) ? $this->post['parent'] : NULL;
      if ($parent && $parent == $this->post["parent_id_entity"]) {
        $parent = NULL;
      }
      if (!empty($field_settings['parent'])) {
        $entity->set($field_settings['parent'], $parent);
      }
      // Set open.
      $open = !empty($this->post['open']) ? $this->post['open'] : 0;
      if (!empty($field_settings['open'])) {
        $entity->set($field_settings['open'], $open);
      }
      // Set owner.
      if (!empty($field_settings["owner"])) {
        $owner = [];
        $this->post['owner'] = json_decode($this->post['owner']);
        if (is_array($this->post['owner'])) {
          foreach ($this->post['owner'] as $owner_id) {
            if (!empty($owner_id)) {
              $owner[] = ['target_id' => $owner_id];
            }
          }
        }
        else {
          $owner = [['target_id' => $this->post['owner']]];
        }
        $entity->set($field_settings['owner'], $owner);
      }
      // Set resource.
      if (!empty($field_settings["custom_resource"])) {
        foreach ($field_settings["custom_resource"] as $field_resource) {
          if ($field_settings["creator"] == $field_resource || $field_settings["owner"] == $field_resource) {
            continue;
          }
          $resource = [];
          $this->post[$field_resource] = json_decode($this->post[$field_resource]);
          if (is_array($this->post[$field_resource])) {
            foreach ($this->post[$field_resource] as $resource_id) {
              if (!empty($resource_id)) {
                $resource[] = ['target_id' => $resource_id];
              }
            }
          }
          else {
            $resource = [['target_id' => $this->post[$field_resource]]];
          }
          $entity->set($field_resource, $resource);
        }
      }
      // Set custom field.
      if (!empty($field_settings["custom_field"]) && !empty($this->post[$field_settings["custom_field"]])) {
        $entity->set($field_settings['custom_field'], $this->post[$field_settings["custom_field"]]);
      }
      // Set creator.
      if (!empty($field_settings["creator"]) && $action == 'inserted') {
        $entity->set($field_settings["creator"], $this->currentUser()->id());
      }
      if ($action == 'inserted') {
        $entity->isNew();
      }
      if ($entity->getEntityTypeId() != 'paragraph') {
        $entity->save();
      }
      else {
        if (!empty($entity_parent)) {
          $entity->save();
        };
      }

      if ($action == 'inserted') {
        $output['tid'] = $entity->id();
        if (!empty($entity_parent) && !empty($parent_field)) {
          $fieldValue = $entity_parent->get($parent_field)->getValue();
          $fieldValue[] = [
            'target_id' => $entity->id(),
            'target_revision_id' => $entity->getRevisionId(),
          ];
          $entity_parent->set($parent_field, $fieldValue);
          $entity_parent->save();
        }
      }
    }

    return $output;
  }

  /**
   * {@inheritDoc}
   */
  public function actionLinks($action, $entityType, $field_settings) {
    $output = ['action' => $action];
    if (!empty($this->post['isImport'])) {
      return new JsonResponse($output);
    }
    $source = !empty($this->post['source']) ? $this->post['source'] : FALSE;
    $target = !empty($this->post['target']) ? $this->post['target'] : FALSE;
    $type = !empty($this->post['type']) ? $this->post['type'] : FALSE;
    $lag = !empty($this->post['lag']) ? $this->post['lag'] : NULL;
    if ($action == 'deleted') {
      [$source, $target] = explode('-', $this->post['id']);
    }
    if (!$source) {
      return FALSE;
    }
    $entity = $entityType->load($source);
    $linkValues = $entity->get($field_settings['links'])->getValue();
    if ($action == 'inserted' || !empty($this->post['isImport'])) {
      if ($field_settings['field_type_links'] == "triples_field") {
        $linkValues[] = [
          'first' => $target,
          'second' => $type,
          'third' => $lag,
        ];
      }
      else {
        $linkValues[] = ['first' => $target, 'second' => $type];
      }
    }
    if ($action == 'updated') {
      foreach ($linkValues as $delta => $linkValue) {
        if ($linkValue['first'] == $target) {
          if ($field_settings['field_type_links'] == "triples_field") {
            $linkValues[$delta] = [
              'first' => $target,
              'second' => $type,
              'third' => $lag,
            ];
          }
          else {
            $linkValues[$delta] = [
              'first' => $target,
              'second' => $type,
            ];
          }
        }
      }
    }
    if ($action == 'deleted') {
      foreach ($linkValues as $delta => $linkValue) {
        if ($linkValue['first'] == $target) {
          unset($linkValues[$delta]);
        }
      }
    }
    $entity->set($field_settings['links'], $linkValues);
    $entity->save();
    $storage = $this->entityTypeManager->getStorage($entity->getEntityTypeId());
    $storage->resetCache([$entity->id()]);

    return new JsonResponse($output);
  }

}
