<?php

namespace Drupal\gantt\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Paragraph Edit Form class.
 */
class EditComponentForm extends ContentEntityForm {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'paragraphs_edit_form';
  }

  /**
   * {@inheritdoc}
   */
  public function getBaseFormId() {
    return $this->getFormId();
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $paragraph = 0) {
    if (empty($this->entity)) {
      $this->setEntity($paragraph);
    }
    $parent = parent::buildForm($form, $form_state);

    $route_params = ['paragraph' => $paragraph->id()];
    $route_info = Url::fromRoute('entity.paragraphs_item.delete_form', $route_params);
    if ($this->getRequest()->query->has('destination')) {
      $query = $route_info->getOption('query');
      $query['destination'] = $this->getRequest()->query->get('destination');
      $route_info->setOption('query', $query);
    }
    $parent['actions']['delete'] = [
      '#type' => 'link',
      '#title' => $this->t('Delete'),
      '#access' => $this->entity->access('delete'),
      '#attributes' => [
        'class' => ['button', 'btn btn-danger', 'button--danger'],
      ],
      '#url' => $route_info,
    ];
    return $parent;
  }

  /**
   * The _title_callback for the paragraphs_item.view route.
   *
   * @param mixed|null $paragraph
   *   The current paragraphs_item.
   *
   * @return string
   *   The page title.
   */
  public function pageTitle(mixed $paragraph = NULL): string {
    return $this->entityRepository->getTranslationFromContext($paragraph)->label() . ' #' . $paragraph->id();
  }

}
